"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// available decoders
const codabar = require("./codabar");
const code128 = require("./code-128");
const code39 = require("./code-39");
const code93 = require("./code-93");
const code2of5 = require("./code2of5");
const ean = require("./ean");
const adaptiveThreshold_1 = require("./utilities/adaptiveThreshold");
const combineAllPossible_1 = require("./utilities/combineAllPossible");
const getImageDataFromSource_1 = require("./utilities/getImageDataFromSource");
const getLines_1 = require("./utilities/getLines");
const isTestEnv = process && process.env.NODE_ENV === 'test';
var BARCODE_DECODERS;
(function (BARCODE_DECODERS) {
    BARCODE_DECODERS["code-128"] = "code-128";
    BARCODE_DECODERS["code-2of5"] = "code-2of5";
    BARCODE_DECODERS["code-39"] = "code-39";
    BARCODE_DECODERS["code-93"] = "code-93";
    BARCODE_DECODERS["ean-13"] = "ean-13";
    BARCODE_DECODERS["ean-8"] = "ean-8";
    BARCODE_DECODERS["codabar"] = "codabar";
})(BARCODE_DECODERS = exports.BARCODE_DECODERS || (exports.BARCODE_DECODERS = {}));
function isImageLike(object) {
    return object.data && object.width && object.height;
}
async function javascriptBarcodeReader({ image, barcode, barcodeType, options, }) {
    let decoder;
    switch (barcode) {
        case BARCODE_DECODERS.codabar:
            decoder = codabar.decoder;
            break;
        case BARCODE_DECODERS['code-128']:
            decoder = code128.decoder;
            break;
        case BARCODE_DECODERS['code-39']:
            decoder = code39.decoder;
            break;
        case BARCODE_DECODERS['code-93']:
            decoder = code93.decoder;
            break;
        case BARCODE_DECODERS['code-2of5']:
            decoder = code2of5.decoder;
            break;
        case BARCODE_DECODERS['ean-13']:
            decoder = ean.decoder;
            barcodeType = '13';
            break;
        case BARCODE_DECODERS['ean-8']:
            decoder = ean.decoder;
            barcodeType = '8';
            break;
        default:
            throw new Error(`Invalid barcode specified. Available decoders: ${BARCODE_DECODERS}.`);
    }
    const useSinglePass = isTestEnv || (options && options.singlePass) || false;
    const { data, width, height } = isImageLike(image) ? image : await getImageDataFromSource_1.getImageDataFromSource(image);
    const channels = data.length / (width * height);
    let finalResult = '';
    // apply adaptive threshold
    if (options && options.useAdaptiveThreshold) {
        adaptiveThreshold_1.applyAdaptiveThreshold(data, width, height);
    }
    // check points for barcode location
    const searchPoints = [5, 6, 4, 7, 3, 8, 2, 9, 1];
    const searchLineStep = Math.round(height / searchPoints.length);
    const rowsToScan = Math.min(2, height);
    for (let i = 0; i < searchPoints.length; i += 1) {
        const start = channels * width * Math.floor(searchLineStep * searchPoints[i]);
        const end = start + rowsToScan * channels * width;
        const lines = getLines_1.getLines(data.slice(start, end), width, rowsToScan);
        if (lines.length === 0) {
            if (useSinglePass || i === searchPoints.length - 1) {
                throw new Error('Failed to detect lines in the image!');
            }
            continue;
        }
        // Run the decoder
        const result = decoder(lines, barcodeType);
        if (!result)
            continue;
        else if (useSinglePass || !result.includes('?'))
            return result;
        finalResult = combineAllPossible_1.combineAllPossible(finalResult, result);
        if (!finalResult.includes('?'))
            return finalResult;
    }
    return finalResult;
}
exports.javascriptBarcodeReader = javascriptBarcodeReader;
//# sourceMappingURL=index.js.map